package com.centeractive.licenseclient.updatecheck;

import java.awt.Desktop;
import java.net.URI;

import org.apache.log4j.Logger;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;

import com.centeractive.licenseclient.Utils;
import com.centeractive.licenseclient.ProductInformation;
import com.centeractive.licenseclient.swt.SWTDialogFactory;

public class UpdateCheckDialog extends Dialog {

    private final static Logger log = Logger.getLogger(UpdateCheckDialog.class);

    private static final int MORE_INFO_ID = IDialogConstants.CLIENT_ID + 1;
    private final UpdateCheckResponse updateCheckResponse;
    private final ProductInformation productInformation;

    protected UpdateCheckDialog(ProductInformation productInfo, Shell parentShell,
            UpdateCheckResponse updateCheckResponse) {
        super(parentShell);
        this.updateCheckResponse = updateCheckResponse;
        this.productInformation = productInfo;
    }

    @Override
    protected Control createDialogArea(Composite parent) {
        Composite composite = (Composite) super.createDialogArea(parent);

        String productName = productInformation.getProductName();

        if (updateCheckResponse.existsNewerVersion(productInformation.getReleaseVersionDate())) {
            createDialogAreaNewerVersionExists(composite, productName);
        } else {
            createDialogAreaRunningNewestVersion(composite, productName);
        }

        return composite;
    }

    private void createDialogAreaRunningNewestVersion(Composite composite, String productName) {
        final Group defaultGroup = new Group(composite, SWT.NONE);
        final GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
        defaultGroup.setLayoutData(gridData);
        defaultGroup.setLayout(new GridLayout(2, true));
        SWTDialogFactory.createLabel(defaultGroup,
                String.format("You have already the newest version of %s installed.", productName));
    }

    private void createDialogAreaNewerVersionExists(Composite composite, String productName) {
        final Group defaultGroup = new Group(composite, SWT.NONE);
        final GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
        defaultGroup.setLayoutData(gridData);
        defaultGroup.setLayout(new GridLayout());

        String currentVersion = productInformation.getReleaseVersion();
        String currentVersionDate = Utils.createISOStringFromDate(productInformation.getReleaseVersionDate());
        String newestVersion = updateCheckResponse.getNewestReleaseVersion();
        String newestVersionDate = Utils.createISOStringFromDate(updateCheckResponse.getNewestReleaseVersionDate());

        SWTDialogFactory.createLabel(defaultGroup, String.format("A new version of %s is available.", productName));
        SWTDialogFactory.createLabel(defaultGroup,
                String.format("Current version: \t%s (%s)", currentVersion, currentVersionDate));
        SWTDialogFactory.createLabel(defaultGroup,
                String.format("New version: \t%s (%s)", newestVersion, newestVersionDate));
        SWTDialogFactory.createLabel(defaultGroup,
                "Press More Info... to open a web page where you can download the newest version!");
        SWTDialogFactory.createLabel(defaultGroup,
                "You can disable the automatic update check on startup in the preferences.");
    }

    @Override
    protected void createButtonsForButtonBar(Composite parent) {
        if (updateCheckResponse.existsNewerVersion(productInformation.getReleaseVersionDate())) {
            createButton(parent, MORE_INFO_ID, "More Info...", true);
            createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
        } else {
            createButton(parent, IDialogConstants.CLOSE_ID, IDialogConstants.CLOSE_LABEL, true);
        }
    }

    @Override
    protected void buttonPressed(int buttonId) {
        super.buttonPressed(buttonId);

        if (buttonId == IDialogConstants.CLOSE_ID) {
            close();
        } else if (buttonId == MORE_INFO_ID) {
            openDownloadUrlInBrowser();
            close();
        }
    }

    private void openDownloadUrlInBrowser() {
        try {
            Desktop.getDesktop().browse(new URI(updateCheckResponse.getDownloadUrl()));
        } catch (Exception e1) {
            log.warn("Could not open default browser.", e1);
        }
    }

    @Override
    protected void configureShell(Shell newShell) {
        super.configureShell(newShell);
        newShell.setText("Update Info");
    }

}
