package com.centeractive.licenseclient.updatecheck;

import java.util.Date;

import org.eclipse.swt.widgets.Shell;

import com.centeractive.licenseclient.Utils;
import com.centeractive.licenseclient.LicenseServerClient;
import com.centeractive.licenseclient.ProductInformation;
import com.centeractive.licenseclient.RuntimeLicenseInformation;
import com.centeractive.licenseclient.SimpleLicenseServerResponse;

public class UpdateChecker {

    private final Shell shell;
    private final ProductInformation productInformation;
    private final RuntimeLicenseInformation licenseInfo;

    public UpdateChecker(ProductInformation productInformation, RuntimeLicenseInformation licenseInfo, Shell shell) {
        this.productInformation = productInformation;
        this.licenseInfo = licenseInfo;
        this.shell = shell;
    }

    public void checkUpdate(boolean showMessageWithCurrentVersionWhenVersionIsCurrent) {
        String registrationKey = licenseInfo.getRegistrationKeyUuid();
        String releaseVersion = productInformation.getReleaseVersion();
        Date releaseDate = productInformation.getReleaseVersionDate();
        UpdateCheckRequest updateCheckRequest = new UpdateCheckRequest(registrationKey, releaseVersion, releaseDate);

        String updateCheckRequestXml = Utils.marshalXML(UpdateCheckRequest.class, updateCheckRequest);

        SimpleLicenseServerResponse serverResponse = makeUpdateCheckRequest(updateCheckRequestXml);

        if (serverResponse.statusCode == 200) {
            UpdateCheckResponse updateCheckResponse = createUpdateCheckResponse(serverResponse.content);
            showUpdateDialog(showMessageWithCurrentVersionWhenVersionIsCurrent, updateCheckResponse);
        }
    }

    private void showUpdateDialog(boolean showMessageWithCurrentVersionWhenVersionIsCurrent,
            UpdateCheckResponse updateCheckResponse) {
        if (showMessageWithCurrentVersionWhenVersionIsCurrent) {
            showUpateCheckDialog(updateCheckResponse);
        } else {
            Date dateOfRunningVersion = productInformation.getReleaseVersionDate();
            boolean isRunningVersionTheNewest = !updateCheckResponse.existsNewerVersion(dateOfRunningVersion);
            if (!isRunningVersionTheNewest) {
                showUpateCheckDialog(updateCheckResponse);
            }
        }
    }

    private void showUpateCheckDialog(final UpdateCheckResponse updateCheckResponse) {
        // show message in UI-Thread
        shell.getDisplay().asyncExec(new Runnable() {

            @Override
            public void run() {
                UpdateCheckDialog updateCheckDialog = new UpdateCheckDialog(productInformation, shell,
                        updateCheckResponse);
                updateCheckDialog.open();
            }
        });
    }

    private LicenseServerClient createLicenseServerRetrospectiveUpdateCheckClient() {
        LicenseServerClient client = new LicenseServerClient(productInformation.getUpdateCheckUrl());
        return client;
    }

    private SimpleLicenseServerResponse makeUpdateCheckRequest(String xml) {
        LicenseServerClient client = createLicenseServerRetrospectiveUpdateCheckClient();
        SimpleLicenseServerResponse response = client.makeRequest(xml);
        return response;
    }

    private UpdateCheckResponse createUpdateCheckResponse(String message) {
        UpdateCheckResponse updateCheckResponse = Utils.unmarshalXml(UpdateCheckResponse.class, message);
        return updateCheckResponse;
    }
}
